import com.robertpenner.graphics.XColorTransform;
import flash.geom.Transform;

/**
* The ColorTransformClip class extends the MovieClip class adding a collection of utilities
* that allows for more intuitive color manipulation.
* 
* @author Mark Walters
*  2006 DigitalFlipbook
*  2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* Based on work by Robert Penner for the Color class.
*  2002 Robert Penner
* http://www.robertpenner.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 8
*/
class com.robertpenner.display.ColorTransformClip extends MovieClip {
	
	/*
	* CLASS PROPERTIES
	*/
	private static var initialized:Boolean = false;
	private static var mixins:ColorTransformClip = new ColorTransformClip();
	private static var mixInProps:Array = ["setRGB", "getRGB", "setRGBStr", "getRGBStr", "setRGB2", "getRGB2", "setColorTransform", "getColorTransform", "resetColor", "setBrightness", "getBrightness", "setBrightOffset", "getBrightOffset", "setTint", "getTint", "setTint2", "getTint2", "setTintOffset", "getTintOffset", "setTintOffset2", "getTintOffset2", "invertColor", "setNegativeColor", "getNegativeColor", "setRed", "getRed", "setGreen", "getGreen", "setBlue", "getBlue", "setRedPercent", "getRedPercent", "setGreenPercent", "getGreenPercent", "setBluePercent", "getBluePercent", "setRedOffset", "getRedOffset", "setGreenOffset", "getGreenOffset", "setBlueOffset", "getBlueOffset", "setAlphaOffset", "getAlphaOffset", "setRedMultiplier", "getRedMultiplier", "setGreenMultiplier", "getGreenMultiplier", "setBlueMultiplier", "getBlueMultiplier", "setAlphaMultiplier", "getAlphaMultiplier", "colorToString"];
	private var $ct:XColorTransform;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	function ColorTransformClip () {
		super();
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	private function createProp (obj:Object, propName:String, setter:Boolean):Void {
		var p = (propName.charAt(0) == "_") ? propName.charAt(1).toUpperCase() + propName.substr(2) : propName.charAt(0).toUpperCase() + propName.substr(1);
		//--CUSTOM--
		if (p == "Rgb") {
			p = "RGB";
		} else if (p == "RgbStr") {
			p = "RGBStr";
		}
		//--CUSTOM--
		var s = null;
		var g = function(Void) {
			return this["get" + p]();
		}
		if (setter) {
			s = function(val) {
				this["set" + p](val);
			}
		}
		obj.addProperty(propName, g, s);
	}
	
	
	/*
	* *********************************************************
	* PUBLIC METHODS
	* *********************************************************
	* 
	*/
	
	/**
	* Decorates all MovieClips with the methods of the XColorTransform class.
	* 
	* @usage
	* <pre><code>
	* com.robertpenner.display.ColorTransformClip.init();
	* </code></pre>
	*/
	public static function init ():Void {
		if (com.robertpenner.display.ColorTransformClip.initialized) { return; };
		com.robertpenner.display.ColorTransformClip.initialized = true;
		com.robertpenner.display.ColorTransformClip.mixin (MovieClip.prototype);
	}
	
	/**
	* Decorates a particular MovieClip instance with the methods of the XColorTransform class.
	* 
	* @param	obj		The object to mix in.
	* 
	* @usage
	* <pre><code>
	* com.robertpenner.display.ColorTransformClip.mixin(my_mc);
	* </code></pre>
	*/
	public static function mixin (obj:Object):Void {
		var m:Array = mixInProps;
		var l:Number = m.length;

		for (var i:Number = 0; i<l; i++) {
			obj[m[i]] = com.robertpenner.display.ColorTransformClip.mixins[m[i]];
			_global.ASSetPropFlags(obj, m[i], 1);
		}
		
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "rgb", true);
		_global.ASSetPropFlags(obj, "rgb", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "rgbStr", true);
		_global.ASSetPropFlags(obj, "rgbStr", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "brightness", true);
		_global.ASSetPropFlags(obj, "brightness", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "brightOffset", true);
		_global.ASSetPropFlags(obj, "brightOffset", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "negativeColor", true);
		_global.ASSetPropFlags(obj, "negativeColor", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "red", true);
		_global.ASSetPropFlags(obj, "red", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "green", true);
		_global.ASSetPropFlags(obj, "green", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "blue", true);
		_global.ASSetPropFlags(obj, "blue", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "redPercent", true);
		_global.ASSetPropFlags(obj, "redPercent", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "greenPercent", true);
		_global.ASSetPropFlags(obj, "greenPercent", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "bluePercent", true);
		_global.ASSetPropFlags(obj, "bluePercent", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "redOffset", true);
		_global.ASSetPropFlags(obj, "redOffset", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "greenOffset", true);
		_global.ASSetPropFlags(obj, "greenOffset", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "blueOffset", true);
		_global.ASSetPropFlags(obj, "blueOffset", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "alphaOffset", true);
		_global.ASSetPropFlags(obj, "alphaOffset", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "redMultiplier", true);
		_global.ASSetPropFlags(obj, "redMultiplier", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "greenMultiplier", true);
		_global.ASSetPropFlags(obj, "greenMultiplier", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "blueMultiplier", true);
		_global.ASSetPropFlags(obj, "blueMultiplier", 1);
		com.robertpenner.display.ColorTransformClip.mixins.createProp(obj, "alphaMultiplier", true);
		_global.ASSetPropFlags(obj, "alphaMultiplier", 1);
	}
	
	/**
	* Specifies an RGB color for a ColorTransformClip object.
	* 
	* @param	col		The hexadecimal or RGB color to be set.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRGB(0x0000FF);
	* </code></pre>
	*/
	public function setRGB (col:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.rgb = col;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the R+G+B combination currently in use by the ColorTransformClip object.
	* 
	* @return	A number that represents the RGB numeric value for the color specified.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.getRGB());
	* </code></pre>
	*/
	public function getRGB ():Number {
		return this.$ct.rgb;
	}
	
	/**
	* Specifies an RGB color for a ColorTransformClip object using a string hexadecimal value.
	* 
	* @param	hexStr	The string hexadecimal color to be set.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRGBStr("#FFFF00");
	* </code></pre>
	*/
	public function setRGBStr (hexStr:String):Void {
		this.$ct = new XColorTransform();
		this.$ct.setRGBStr(hexStr);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the R+G+B combination currently in use by the ColorTransformClip object as a string hexadecimal value.
	* 
	* @return	The string hexadecimal color.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.getRGBStr());
	* </code></pre>
	*/
	public function getRGBStr ():String {
		return this.$ct.getRGBStr();
	}
	
	/**
	* Specifies an RGB color for a ColorTransformClip object using individual red, green, and blue values.
	* 
	* @param	r		The red color value.
	* @param	g		The green color value.
	* @param	b		The blue color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRGB2(255, 0, 255);
	* </code></pre>
	*/
	public function setRGB2 (r:Number, g:Number, b:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setRGB2(r, g, b);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the R+G+B values currently in use by the ColorTransformClip object as individual red, green, and blue values.
	* 
	* @return	The RBG value object with r, g, and b properties.
	* 
	* @usage
	* <pre><code>
	* var rgb:Object = my_mc.getRGB2();
	* trace (rgb.r);
	* trace (rgb.g);
	* trace (rgb.b);
	* </code></pre>
	*/
	public function getRGB2 ():Object {
		return this.$ct.getRGB2();
	}
	
	/**
	* Sets the color transform for a ColorTransformClip object to its default values.
	* 
	* @usage
	* <pre><code>
	* my_mc.rgb = 0x0000FF;
	* my_mc.resetColor();
	* </code></pre>
	*/
	public function resetColor ():Void {
		this.$ct = new XColorTransform();
		this.$ct.reset();
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Specifies the brightness of a ColorTransformClip object.
	* 
	* @param	bright	A percentage value between -100 and 100. Positive values brighten the color. and negative values darken it.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBrightness(50);
	* </code></pre>
	*/
	public function setBrightness (bright:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setBrightness(bright);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the brightness of a ColorTransformClip object.
	* 
	* @return	The brightness value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBrightness(50);
	* trace (my_mc.getBrightness());
	* </code></pre>
	*/
	public function getBrightness ():Number {
		return this.$ct.getBrightness();
	}
	
	/**
	* Pushes a ColorTransformClip object towards black or white by an absolute amount.
	* 
	* @param	offset	A number between -255 and 255. Positive values push colors towards white; negative values towards black.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBrightOffset(140);
	* </code></pre>
	*/
	public function setBrightOffset (offset:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setBrightOffset(offset);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the brightness offset value of a ColorTransformClip object.
	* 
	* @return	The brightness offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBrightOffset(140);
	* trace (my_mc.getBrightOffset());
	* </code></pre>
	*/
	public function getBrightOffset ():Number {
		return this.$ct.getBrightOffset();
	}
	
	/**
	* Tints a ColorTransformClip object with a color according to a certain percentage.
	* 
	* @param	r		The red color value.
	* @param	g		The green color value.
	* @param	b		The blue color value.
	* @param	percent	The tint percentage.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTint(0, 0, 128, 50);
	* </code></pre>
	*/
	public function setTint (r:Number, g:Number, b:Number, percent:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setTint(r, g, b, percent);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the tint of a ColorTransformClip object.
	* 
	* @return	The tint value object with r, g, b, and percent properties.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTint(0, 0, 128, 50);
	* var tint:Object = my_mc.getTint();
	* trace (tint.r);
	* trace (tint.g);
	* trace (tint.b);
	* trace (tint.percent);
	* </code></pre>
	*/
	public function getTint ():Object {
		return this.$ct.getTint();
	}
	
	/**
	* Tints a ColorTransformClip object with a color according to a certain percentage.
	* 
	* @param	rgb		The rgb value.
	* @param	percent	The tint percentage.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTint2(0x0000FF, 100);
	* </code></pre>
	*/
	public function setTint2 (rgb:Number, percent:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setTint2(rgb, percent);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the tint of a ColorTransformClip object.
	* 
	* @return	The tint value object with rgb and percent properties.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTint2(0x0000FF, 100);
	* var tint:Object = my_mc.getTint2();
	* trace (tint.rgb);
	* trace (tint.percent);
	* </code></pre>
	*/
	public function getTint2 ():Object {
		return this.$ct.getTint2();
	}
	
	/**
	* Tints a ColorTransformClip object with a color according to red, green, and blue values.
	* 
	* @param	r		The red color value.
	* @param	g		The green color value.
	* @param	b		The blue color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTintOffset(0, 0, 128);
	* </code></pre>
	*/
	public function setTintOffset (r:Number, g:Number, b:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setTintOffset(r, g, b);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the tint offset of a ColorTransformClip object.
	* 
	* @return	The tint offset value object with r, g, and b properties.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTintOffset(0, 0, 128);
	* var tint:Object = my_mc.getTintOffset();
	* trace (tint.r);
	* trace (tint.g);
	* trace (tint.b);
	* </code></pre>
	*/
	public function getTintOffset ():Object {
		return this.$ct.getTintOffset();
	}
	
	/**
	* Tints a ColorTransformClip object with a color according to an rgb value.
	* 
	* @param	rgb		The rgb value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTintOffset2(0x000080);
	* </code></pre>
	*/
	public function setTintOffset2 (rgb:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setTintOffset2(rgb);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the tint offset of a ColorTransformClip object.
	* 
	* @return	The tint offset value object with an rgb properties.
	* 
	* @usage
	* <pre><code>
	* my_mc.setTintOffset2(0x000080);
	* trace (my_mc.getTintOffset2());
	* </code></pre>
	*/
	public function getTintOffset2 ():Number {
		return this.$ct.getTintOffset2();
	}
	
	/**
	* Performs a straightforward color inversion.
	* 
	* @usage
	* <pre><code>
	* my_mc.invertColor();
	* </code></pre>
	*/
	public function invertColor ():Void {
		this.$ct = new XColorTransform();
		this.$ct.invert();
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Inverts a ColorTransformClip object by a certain percentage.
	* 
	* @param	percent	The negative percentage.
	* 
	* @usage
	* <pre><code>
	* my_mc.setNegativeColor(60);
	* </code></pre>
	*/
	public function setNegativeColor (percent:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setNegative(percent);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the negative percentage of a ColorTransformClip object.
	* 
	* @return	The negative percentage.
	* 
	* @usage
	* <pre><code>
	* my_mc.setNegativeColor(60);
	* trace (my_mc.getNegativeColor());
	* </code></pre>
	*/
	public function getNegativeColor ():Number {
		return this.$ct.getNegative();
	}
	
	/**
	* Specifies a red color value for a ColorTransformClip object.
	* 
	* @param	amount	The red color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRed(128);
	* </code></pre>
	*/
	public function setRed (amount:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setRed(amount);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the red color value of a ColorTransformClip object.
	* 
	* @return	The red color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRed(128);
	* trace (my_mc.getRed());
	* </code></pre>
	*/
	public function getRed ():Number {
		return this.$ct.getRed();
	}
	
	/**
	* Specifies a green color value for a ColorTransformClip object.
	* 
	* @param	amount	The green color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreen(128);
	* </code></pre>
	*/
	public function setGreen (amount:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setGreen(amount);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the green color value of a ColorTransformClip object.
	* 
	* @return	The green color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreen(128);
	* trace (my_mc.getGreen());
	* </code></pre>
	*/
	public function getGreen ():Number {
		return this.$ct.getGreen();
	}
	
	/**
	* Specifies a blue color value for a ColorTransformClip object.
	* 
	* @param	amount	The blue color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBlue(128);
	* </code></pre>
	*/
	public function setBlue (amount:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setBlue(amount);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the blue color value of a ColorTransformClip object.
	* 
	* @return	The blue color value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBlue(128);
	* trace (my_mc.getBlue());
	* </code></pre>
	*/
	public function getBlue ():Number {
		return this.$ct.getBlue();
	}
	
	/**
	* Specifies a red percentage value for a ColorTransformClip object.
	* 
	* @param	percent	The red percentage value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRedPercent(50);
	* </code></pre>
	*/
	public function setRedPercent (percent:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setRedPercent(percent);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the red percentage value of a ColorTransformClip object.
	* 
	* @return	The red percentage value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRedPercent(50);
	* trace (my_mc.getRedPercent());
	* </code></pre>
	*/
	public function getRedPercent ():Number {
		return this.$ct.getRedPercent();
	}
	
	/**
	* Specifies a green percentage value for a ColorTransformClip object.
	* 
	* @param	percent	The green percentage value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreenPercent(50);
	* </code></pre>
	*/
	public function setGreenPercent (percent:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setGreenPercent(percent);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the green percentage value of a ColorTransformClip object.
	* 
	* @return	The green percentage value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreenPercent(50);
	* trace (my_mc.getGreenPercent());
	* </code></pre>
	*/
	public function getGreenPercent ():Number {
		return this.$ct.getGreenPercent();
	}
	
	/**
	* Specifies a blue percentage value for a ColorTransformClip object.
	* 
	* @param	percent	The blue percentage value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBluePercent(50);
	* </code></pre>
	*/
	public function setBluePercent (percent:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.setBluePercent(percent);
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the blue percentage value of a ColorTransformClip object.
	* 
	* @return	The blue percentage value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBluePercent(50);
	* trace (my_mc.getBluePercent());
	* </code></pre>
	*/
	public function getBluePercent ():Number {
		return this.$ct.getBluePercent();
	}
	
	/**
	* Specifies a red offset value for a ColorTransformClip object.
	* 
	* @param	offset	The red offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRedOffset(-128);
	* </code></pre>
	*/
	public function setRedOffset (offset:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.redOffset = offset;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the red offset value of a ColorTransformClip object.
	* 
	* @return	The red offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRedOffset(-128);
	* trace (my_mc.getRedOffset());
	* </code></pre>
	*/
	public function getRedOffset ():Number {
		return this.$ct.redOffset;
	}
	
	/**
	* Specifies a green offset value for a ColorTransformClip object.
	* 
	* @param	offset	The green offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreenOffset(-128);
	* </code></pre>
	*/
	public function setGreenOffset (offset:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.greenOffset = offset;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the green offset value of a ColorTransformClip object.
	* 
	* @return	The green offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreenOffset(-128);
	* trace (my_mc.getGreenOffset());
	* </code></pre>
	*/
	public function getGreenOffset ():Number {
		return this.$ct.greenOffset;
	}
	
	/**
	* Specifies a blue offset value for a ColorTransformClip object.
	* 
	* @param	offset	The blue offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBlueOffset(-128);
	* </code></pre>
	*/
	public function setBlueOffset (offset:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.blueOffset = offset;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the blue offset value of a ColorTransformClip object.
	* 
	* @return	The blue offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBlueOffset(-128);
	* trace (my_mc.getBlueOffset());
	* </code></pre>
	*/
	public function getBlueOffset ():Number {
		return this.$ct.blueOffset;
	}
	
	/**
	* Specifies an alpha offset value for a ColorTransformClip object.
	* 
	* @param	offset	The alpha offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setAlphaOffset(-128);
	* </code></pre>
	*/
	public function setAlphaOffset (offset:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.alphaOffset = offset;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the alpha offset value of a ColorTransformClip object.
	* 
	* @return	The alpha offset value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setAlphaOffset(-128);
	* trace (my_mc.getAlphaOffset());
	* </code></pre>
	*/
	public function getAlphaOffset ():Number {
		return this.$ct.alphaOffset;
	}
	
	/**
	* Specifies a red multiplier value for a ColorTransformClip object.
	* 
	* @param	multiplier	The red multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRedMultiplier(.5);
	* </code></pre>
	*/
	public function setRedMultiplier (multiplier:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.redMultiplier = multiplier;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the red multiplier value of a ColorTransformClip object.
	* 
	* @return	The red multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setRedMultiplier(.5);
	* trace (my_mc.getRedMultiplier());
	* </code></pre>
	*/
	public function getRedMultiplier ():Number {
		return this.$ct.redMultiplier;
	}
	
	/**
	* Specifies a green multiplier value for a ColorTransformClip object.
	* 
	* @param	multiplier	The green multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreenMultiplier(.5);
	* </code></pre>
	*/
	public function setGreenMultiplier (multiplier:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.greenMultiplier = multiplier;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the green multiplier value of a ColorTransformClip object.
	* 
	* @return	The green multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setGreenMultiplier(.5);
	* trace (my_mc.getGreenMultiplier());
	* </code></pre>
	*/
	public function getGreenMultiplier ():Number {
		return this.$ct.greenMultiplier;
	}
	
	/**
	* Specifies a blue multiplier value for a ColorTransformClip object.
	* 
	* @param	multiplier	The blue multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBlueMultiplier(.5);
	* </code></pre>
	*/
	public function setBlueMultiplier (multiplier:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.blueMultiplier = multiplier;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the blue multiplier value of a ColorTransformClip object.
	* 
	* @return	The blue multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setBlueMultiplier(.5);
	* trace (my_mc.getBlueMultiplier());
	* </code></pre>
	*/
	public function getBlueMultiplier ():Number {
		return this.$ct.blueMultiplier;
	}
	
	/**
	* Specifies an alpha multiplier value for a ColorTransformClip object.
	* 
	* @param	multiplier	The alpha multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setAlphaMultiplier(.5);
	* </code></pre>
	*/
	public function setAlphaMultiplier (multiplier:Number):Void {
		this.$ct = new XColorTransform();
		this.$ct.alphaMultiplier = multiplier;
		this.transform.colorTransform = this.$ct;
	}
	
	/**
	* Returns the alpha multiplier value of a ColorTransformClip object.
	* 
	* @return	The alpha multiplier value.
	* 
	* @usage
	* <pre><code>
	* my_mc.setAlphaMultiplier(.5);
	* trace (my_mc.getAlphaMultiplier());
	* </code></pre>
	*/
	public function getAlphaMultiplier ():Number {
		return this.$ct.alphaMultiplier;
	}
	
	/**
	* Formats and returns a string that describes all of the color properties of the ColorTransformClip object.
	* 
	* @return	A string that lists all of the color properties of the ColorTransformClip object.
	* 
	* @usage
	* <pre><code>
	* trace(my_mc.colorToString());
	* </code></pre>
	*/
	public function colorToString():String {
		return this.$ct.toString();
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The rgb property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.rgb);
	* </code></pre>
	*/
	public function get rgb ():Number {
		return this.getRGB();
	}
	public function set rgb (col:Number):Void {
		this.setRGB(col);
	}
	
	/**
	* The rgbStr property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.rgbStr);
	* </code></pre>
	*/
	public function get rgbStr ():String {
		return this.getRGBStr();
	}
	public function set rgbStr (hexStr:String):Void {
		this.setRGBStr(hexStr);
	}
	
	/**
	* The brightness property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.brightness);
	* </code></pre>
	*/
	public function get brightness ():Number {
		return this.getBrightness();
	}
	public function set brightness (bright:Number):Void {
		this.setBrightness(bright);
	}
	
	/**
	* The brightOffset property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.brightOffset);
	* </code></pre>
	*/
	public function get brightOffset ():Number {
		return this.getBrightOffset();
	}
	public function set brightOffset (offset:Number):Void {
		this.setBrightOffset(offset);
	}
	
	/**
	* The negativeColor property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.negativeColor);
	* </code></pre>
	*/
	public function get negativeColor ():Number {
		return this.getNegativeColor();
	}
	public function set negativeColor (percent:Number):Void {
		this.setNegativeColor(percent);
	}
	
	/**
	* The red property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.red);
	* </code></pre>
	*/
	public function get red ():Number {
		return this.getRed();
	}
	public function set red (amount:Number):Void {
		this.setRed(amount);
	}
	
	/**
	* The green property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.green);
	* </code></pre>
	*/
	public function get green ():Number {
		return this.getGreen();
	}
	public function set green (amount:Number):Void {
		this.setGreen(amount);
	}
	
	/**
	* The blue property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.blue);
	* </code></pre>
	*/
	public function get blue ():Number {
		return this.getBlue();
	}
	public function set blue (amount:Number):Void {
		this.setBlue(amount);
	}
	
	/**
	* The redPercent property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.redPercent);
	* </code></pre>
	*/
	public function get redPercent ():Number {
		return this.getRedPercent();
	}
	public function set redPercent (percent:Number):Void {
		this.setRedPercent(percent);
	}
	
	/**
	* The greenPercent property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.greenPercent);
	* </code></pre>
	*/
	public function get greenPercent ():Number {
		return this.getGreenPercent();
	}
	public function set greenPercent (percent:Number):Void {
		this.setGreenPercent(percent);
	}
	
	/**
	* The bluePercent property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.bluePercent);
	* </code></pre>
	*/
	public function get bluePercent ():Number {
		return this.getBluePercent();
	}
	public function set bluePercent (percent:Number):Void {
		this.setBluePercent(percent);
	}
	
	/**
	* The redOffset property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.redOffset);
	* </code></pre>
	*/
	public function get redOffset ():Number {
		return this.getRedOffset();
	}
	public function set redOffset (offset:Number):Void {
		this.setRedOffset(offset);
	}
	
	/**
	* The greenOffset property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.greenOffset);
	* </code></pre>
	*/
	public function get greenOffset ():Number {
		return this.getGreenOffset();
	}
	public function set greenOffset (offset:Number):Void {
		this.setGreenOffset(offset);
	}
	
	/**
	* The blueOffset property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.blueOffset);
	* </code></pre>
	*/
	public function get blueOffset ():Number {
		return this.getBlueOffset();
	}
	public function set blueOffset (offset:Number):Void {
		this.setBlueOffset(offset);
	}
	
	/**
	* The alphaOffset property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.alphaOffset);
	* </code></pre>
	*/
	public function get alphaOffset ():Number {
		return this.getAlphaOffset();
	}
	public function set alphaOffset (offset:Number):Void {
		this.setAlphaOffset(offset);
	}
	
	/**
	* The redMultiplier property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.redMultiplier);
	* </code></pre>
	*/
	public function get redMultiplier ():Number {
		return this.getRedMultiplier();
	}
	public function set redMultiplier (multiplier:Number):Void {
		this.setRedMultiplier(multiplier);
	}
	
	/**
	* The greenMultiplier property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.greenMultiplier);
	* </code></pre>
	*/
	public function get greenMultiplier ():Number {
		return this.getGreenMultiplier();
	}
	public function set greenMultiplier (multiplier:Number):Void {
		this.setGreenMultiplier(multiplier);
	}
	
	/**
	* The blueMultiplier property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.blueMultiplier);
	* </code></pre>
	*/
	public function get blueMultiplier ():Number {
		return this.getBlueMultiplier();
	}
	public function set blueMultiplier (multiplier:Number):Void {
		this.setBlueMultiplier(multiplier);
	}
	
	/**
	* The alphaMultiplier property of the ColorTransformClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.alphaMultiplier);
	* </code></pre>
	*/
	public function get alphaMultiplier ():Number {
		return this.getAlphaMultiplier();
	}
	public function set alphaMultiplier (multiplier:Number):Void {
		this.setAlphaMultiplier(multiplier);
	}
	
}
