/**
* The DrawingClip class extends the MovieClip class adding advanced drawing utilities.
* 
* @author Robert Penner (AS 1.0)
*  2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
*  2006 DigitalFlipbook
*  2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.display.DrawingClip extends MovieClip {
	
	/*
	* CLASS PROPERTIES
	*/
	private static var initialized:Boolean = false;
	private static var mixins:DrawingClip = new DrawingClip();
	private static var mixInProps:Array = ["$moveTo", "moveTo", "$lineTo", "lineTo", "$curveTo", "curveTo", "$clear", "clear", "setXPen", "getXPen", "setYPen", "getYPen", "getXPenStart", "getYPenStart", "drawLine", "drawTri", "drawQuad", "drawRect", "drawRectRel", "drawRectCent", "drawSquare", "drawSquareCent", "drawDot", "drawPoly", "drawRegPoly", "drawOval", "drawCircle", "curveThruTo", "drawCurve3Pts", "createBezier", "intersect2Lines", "midLine", "bezierSplit", "drawBezier", "drawBezierPts", "curveToCubic", "curveToCubicPts"];
	private var $moveTo:Function = MovieClip.prototype.moveTo;
	private var $lineTo:Function = MovieClip.prototype.lineTo;
	private var $curveTo:Function = MovieClip.prototype.curveTo;
	private var $clear:Function = MovieClip.prototype.clear;
	private var $xpen:Number;
	private var $ypen:Number;
	private var $xpenStart:Number;
	private var $ypenStart:Number;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	function DrawingClip () {
		super();
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	private function createProp (obj:Object, propName:String, setter:Boolean):Void {
		var p = (propName.charAt(0) == "_") ? propName.charAt(1).toUpperCase() + propName.substr(2) : propName.charAt(0).toUpperCase() + propName.substr(1);
		//--CUSTOM--
		if (p == "Xpen") {
			p = "XPen";
		} else if (p == "Ypen") {
			p = "YPen";
		} else if (p == "XpenStart") {
			p = "XPenStart";
		} else if (p == "YpenStart") {
			p = "YPenStart";
		}
		//--CUSTOM--
		var s = null;
		var g = function(Void) {
			return this["get" + p]();
		}
		if (setter) {
			s = function(val) {
				this["set" + p](val);
			}
		}
		obj.addProperty(propName, g, s);
	}
	
	private function createBezier (a1:Object, c1:Object, c2:Object, a2:Object, t:Number):Void {
		var s:Object = this.intersect2Lines (a1, c1, c2, a2);
		var dx:Number = (a1.x + a2.x + s.x * 4 - (c1.x + c2.x) * 3) * .125;
		var dy:Number = (a1.y + a2.y + s.y * 4 - (c1.y + c2.y) * 3) * .125;
		if (dx*dx + dy*dy > t) {
			var halves:Object = this.bezierSplit (a1, c1, c2, a2);
			var b0 = halves.b0;
			var b1 = halves.b1;
			this.createBezier (a1,     b0.c1, b0.c2, b0.a2, t);
			this.createBezier (b1.a1,  b1.c1, b1.c2, a2,    t);
		} else {
			this.curveTo (s.x, s.y, a2.x, a2.y);
		}
	}
	
	private function intersect2Lines (p1:Object, p2:Object, p3:Object, p4:Object):Object {
		var x1:Number = p1.x;
		var y1:Number = p1.y;
		var x4:Number = p4.x;
		var y4:Number = p4.y;

		var dx1:Number = p2.x - x1;
		var dx2:Number = p3.x - x4;
		if (!(dx1 || dx2)) return NaN;
		
		var m1:Number = (p2.y - y1) / dx1;
		var m2:Number = (p3.y - y4) / dx2;
		
		if (!dx1) {
			// infinity
			return { x:x1, y:(m2 * (x1 - x4) + y4) };

		} else if (!dx2) {
			// infinity
			return { x:x4, y:(m1 * (x4 - x1) + y1) };
		}
		var xInt:Number = (-m2 * x4 + y4 + m1 * x1 - y1) / (m1 - m2);
		var yInt:Number = m1 * (xInt - x1) + y1;
		return { x:xInt, y:yInt };
	}
	
	private function midLine (a:Object, b:Object):Object {
		return { x:(a.x + b.x)/2, y:(a.y + b.y)/2 };
	}
	
	private function bezierSplit (p0:Object, p1:Object, p2:Object, p3:Object):Object {
		var m = this.midLine;
		var p01:Object = m (p0, p1);
		var p12:Object = m (p1, p2);
		var p23:Object = m (p2, p3);
		var p02:Object = m (p01, p12);
		var p13:Object = m (p12, p23);
		var p03:Object = m (p02, p13);
		return {
			b0:{a1:p0,  c1:p01, c2:p02, a2:p03},
			b1:{a1:p03, c1:p13, c2:p23, a2:p3 }
		}
	}
	
	
	/*
	* *********************************************************
	* PUBLIC METHODS
	* *********************************************************
	* 
	*/
	
	/**
	* Decorates all MovieClips with the methods of this class.
	* 
	* @usage
	* <pre><code>
	* com.robertpenner.display.DrawingClip.init();
	* </code></pre>
	*/
	public static function init ():Void {
		if (com.robertpenner.display.DrawingClip.initialized) { return; };
		com.robertpenner.display.DrawingClip.initialized = true;
		com.robertpenner.display.DrawingClip.mixin (MovieClip.prototype);
	}
	
	/**
	* Decorates a particular MovieClip instance with the methods of this class.
	* 
	* @param	obj		The object to mix in.
	* 
	* @usage
	* <pre><code>
	* com.robertpenner.display.DrawingClip.mixin(my_mc);
	* </code></pre>
	*/
	public static function mixin (obj:Object):Void {
		var m:Array = mixInProps;
		var l:Number = m.length;

		for (var i:Number = 0; i<l; i++) {
			obj[m[i]] = com.robertpenner.display.DrawingClip.mixins[m[i]];
			_global.ASSetPropFlags(obj, m[i], 1);
		}
		
		com.robertpenner.display.DrawingClip.mixins.createProp(obj, "xpen", true);
		_global.ASSetPropFlags(obj, "xpen", 1);
		com.robertpenner.display.DrawingClip.mixins.createProp(obj, "ypen", true);
		_global.ASSetPropFlags(obj, "ypen", 1);
		com.robertpenner.display.DrawingClip.mixins.createProp(obj, "xpenStart", false);
		_global.ASSetPropFlags(obj, "xpenStart", 1);
		com.robertpenner.display.DrawingClip.mixins.createProp(obj, "ypenStart", false);
		_global.ASSetPropFlags(obj, "ypenStart", 1);
	}
	
	public function moveTo (x:Number, y:Number):Void {
		this.$moveTo(x, y);
		this.$xpen = this.$xpenStart = x;
		this.$ypen = this.$ypenStart = y;
	}
	
	public function lineTo (x:Number, y:Number):Void {
		this.$lineTo(x, y);
		this.$xpen = x;
		this.$ypen = y;
	}
	
	public function curveTo (cx:Number, cy:Number, ax:Number, ay:Number):Void {
		this.$curveTo(cx, cy, ax, ay);
		this.$xpen = ax;
		this.$ypen = ay;
	}
	
	public function clear ():Void {
		this.$clear();
		this.$xpen = this.$ypen = this.$xpenStart = this.$ypenStart = 0;
	}
	
	/**
	* Sets the x coordinate of the drawing pen.
	* 
	* @param	x		The x coordinate of the drawing pen.
	* 
	* @usage
	* <pre><code>
	* my_mc.setXPen(50);
	* </code></pre>
	*/
	public function setXPen (x:Number):Void {
		this.$xpen = x;
		this.moveTo(x, this.$ypen);
	}
	
	/**
	* Returns the x coordinate of the drawing pen.
	* 
	* @return	The x coordinate of the drawing pen.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.getXPen());
	* </code></pre>
	*/
	public function getXPen ():Number {
		if (this.$xpen == undefined) this.$xpen = 0;
		return this.$xpen;
	}
	
	/**
	* Sets the y coordinate of the drawing pen.
	* 
	* @param	y		The y coordinate of the drawing pen.
	* 
	* @usage
	* <pre><code>
	* my_mc.setYPen(50);
	* </code></pre>
	*/
	public function setYPen (y:Number):Void {
		this.$ypen = y;
		this.moveTo(this.$xpen, y);
	}
	
	/**
	* Returns the y coordinate of the drawing pen.
	* 
	* @return	The y coordinate of the drawing pen.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.getYPen());
	* </code></pre>
	*/
	public function getYPen ():Number {
		if (this.$ypen == undefined) this.$ypen = 0;
		return this.$ypen;
	}
	
	/**
	* Returns the x coordinate of the point where a new subpath was started.
	* 
	* @return	The x coordinate of the point where a new subpath was started.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.getXPenStart());
	* </code></pre>
	*/
	public function getXPenStart ():Number {
		if (this.$xpenStart == undefined) this.$xpenStart = 0;
		return this.$xpenStart;
	}
	
	/**
	* Returns the y coordinate of the point where a new subpath was started.
	* 
	* @return	The y coordinate of the point where a new subpath was started.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.getYPenStart());
	* </code></pre>
	*/
	public function getYPenStart ():Number {
		if (this.$ypenStart == undefined) this.$ypenStart = 0;
		return this.$ypenStart;
	}
	
	/**
	* Draws a line between two points using the current line style.
	* 
	* @param	x1		The x position of the first point.
	* @param	y1		The y position of the first point.
	* @param	x2		The x position of the second point.
	* @param	y2		The y position of the second point.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(4, 0x0000FF, 60);
	* my_mc.drawLine(0, 0, 50, 50);
	* </code></pre>
	*/
	public function drawLine (x1:Number, y1:Number, x2:Number, y2:Number):Void {
		with (this) {
			moveTo (x1, y1);
			lineTo (x2, y2);
		}
	}
	
	/**
	* Draws a triangle defined by three corner points.
	* 
	* @param	p1		The first corner point of the triangle.
	* @param	p2		The second corner point of the triangle.
	* @param	p3		The third corner point of the triangle.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(4, 0x0000FF, 60);
	* var a:Object = {x:20, y:100};
	* var b:Object = {x:90, y:0};
	* var c:Object = {x:200, y:50};
	* my_mc.drawTri(a, b, c);
	* </code></pre>
	*/
	public function drawTri (p1:Object, p2:Object, p3:Object):Void {
		with (this) {
			moveTo (p1.x, p1.y);
			lineTo (p2.x, p2.y);
			lineTo (p3.x, p3.y);
			lineTo (p1.x, p1.y);
		}
	}
	
	/**
	* Draws a quadrilateral defined by four corner points.
	* 
	* @param	p1		The first corner point of the quadrilateral.
	* @param	p2		The second corner point of the quadrilateral.
	* @param	p3		The third corner point of the quadrilateral.
	* @param	p4		The fourth corner point of the quadrilateral.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0x00CC22, 40);
	* my_mc.lineStyle(2, 0x330000);
	* var a:Object = {x:0, y:20};
	* var b:Object = {x:10, y:80};
	* var c:Object = {x:100, y:120};
	* var d:Object = {x:150, y:0};
	* my_mc.drawQuad(a, b, c, d);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawQuad (p1:Object, p2:Object, p3:Object, p4:Object):Void {
		with (this) {
			moveTo (p1.x, p1.y);
			lineTo (p2.x, p2.y);
			lineTo (p3.x, p3.y);
			lineTo (p4.x, p4.y);
			lineTo (p1.x, p1.y);
		}
	}
	
	/**
	* Draws a rectangle defined by two corner points.
	* 
	* @param	x1		The x position of the first corner point.
	* @param	y1		The y position of the first corner point.
	* @param	x2		The x position of the second corner point.
	* @param	y2		The y position of the second corner point.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0x33DD00, 60);
	* my_mc.drawRect(10, 10, 240, 150);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawRect (x1:Number, y1:Number, x2:Number, y2:Number):Void {
		with (this) {
			moveTo (x1, y1);
			lineTo (x2, y1);
			lineTo (x2, y2);
			lineTo (x1, y2);
			lineTo (x1, y1);
		}
	}
	
	/**
	* Draws a rectangle defined by relative coordinates, a starting point and a width and height.
	* 
	* @param	x		The x position of the starting point.
	* @param	y		The y position of the starting point.
	* @param	w		The width of the rectangle.
	* @param	h		The height of the rectangle.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0x0000FF, 40);
	* my_mc.drawRectRel(60, 60, 20, 30);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawRectRel (x:Number, y:Number, w:Number, h:Number):Void {
		this.drawRect(x, y, x + w, y + h);
	}
	
	/**
	* Draws a rectangle defined by a center point and a width and height.
	* 
	* @param	x		The x position of the center point.
	* @param	y		The y position of the center point.
	* @param	w		The width of the rectangle.
	* @param	h		The height of the rectangle.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(2, 0x00FF00);
	* my_mc.drawRectCent(150, 100, 70, 10);
	* </code></pre>
	*/
	public function drawRectCent (x:Number, y:Number, w:Number, h:Number):Void {
		this.drawRect (x - w/2, y - h/2, x + w/2, y + h/2);
	}
	
	/**
	* Draws a square defined by a starting point and a width.
	* 
	* @param	x		The x position of the starting point.
	* @param	y		The y position of the starting point.
	* @param	w		The width of the square.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(2, 0x00FF00);
	* my_mc.drawSquare(250, 10, 50);
	* </code></pre>
	*/
	public function drawSquare (x:Number, y:Number, w:Number):Void {
		this.drawRect (x, y, x + w, y + w);
	}
	
	/**
	* Draws a square defined by a center point and a width.
	* 
	* @param	x		The x position of the center point.
	* @param	y		The y position of the center point.
	* @param	w		The width of the square.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(2, 0x00FF00);
	* my_mc.drawSquare(250, 10, 50);
	* </code></pre>
	*/
	public function drawSquareCent (x:Number, y:Number, w:Number):Void {
		var r:Number = w / 2;
		this.drawRect (x - r, y - r, x + r, y + r);
	}
	
	/**
	* Draws a one-pixel wide dot defined by a point.
	* 
	* @param	x		The x position of the point.
	* @param	y		The y position of the point.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0xFF0000);
	* my_mc.drawDot(130, 100);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawDot (x:Number, y:Number):Void {
		this.drawRect (x - .5, y - .5, x + .5, y + .5);
	}
	
	/**
	* Draws a polygon defined by a collection of points.
	* 
	* @param	pts		An array of point objects, each point containing x and y properties.
	* 
	* @usage
	* <pre><code>
	* var pts:Array = [];
	* var numPoints:Number = 50;
	* while (numPoints--) {
	* 	pts.push({x:Math.random()*300, y:Math.random()*300});
	* }
	* my_mc.beginFill(0x339933);
	* my_mc.drawPoly(pts);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawPoly (pts:Array):Void {
		this.moveTo (pts[0].x, pts[0].y);
		var i:Number = pts.length;
		while (i--) this.lineTo (pts[i].x, pts[i].y);
	}
	
	/**
	* Draws a regular polygon (a polygon with equal angles and sides).
	* 
	* @param	x			The x position of the center point.
	* @param	y			The y position of the center point.
	* @param	r			The radius from the center to one of the polygon's corners.
	* @param	numPts		The number of points (and sides) of the polygon.
	* @param	rotation	The angle (in degrees) to rotate the polygon.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0xFF0000, 70);
	* my_mc.drawRegPoly(100, 100, 90, 5, 15);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawRegPoly (x:Number, y:Number, r:Number, numPts:Number, rotation:Number):Void {
		if (!rotation) rotation = 0;
		var angle:Number = (-90 + rotation) * (Math.PI/180);
		var pts:Array = [];
		var px:Number;
		var py:Number;
		var dAngle:Number = 2 * Math.PI / numPts;
		var cos:Function = Math.cos;
		var sin:Function = Math.sin;
		while (numPts--) {
			angle += dAngle;
			px = r * cos (angle) + x;
			py = r * sin (angle) + y;
			pts.push ({x:px, y:py});	
		}
		this.drawPoly(pts);
	}
	
	/**
	* Draws an oval.
	* 
	* @param	x		The x position of the center point.
	* @param	y		The y position of the center point.
	* @param	rx		The horizontal radius from the center to the edge of the oval.
	* @param	ry		The vertical radius from the center to the edge of the oval.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0x0000FF, 40);
	* my_mc.drawOval(80, 100, 50, 75);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawOval (x:Number, y:Number, rx:Number, ry:Number):Void {
		with (this) {
			moveTo (x+rx, y);
			curveTo (rx+x, 0.4142*ry+y, 0.7071*rx+x, 0.7071*ry+y);
			curveTo (0.4142*rx+x, ry+y, x, ry+y);
			curveTo (-0.4142*rx+x, ry+y, -0.7071*rx+x, 0.7071*ry+y);
			curveTo (-rx+x, 0.4142*ry+y, -rx+x, y);
			curveTo (-rx+x, -0.4142*ry+y, -0.7071*rx+x, -0.7071*ry+y);
			curveTo (-0.4142*rx+x, -ry+y, x, -ry+y);
			curveTo (0.4142*rx+x, -ry+y, 0.7071*rx+x, -0.7071*ry+y);
			curveTo (rx+x, -0.4142*ry+y, rx+x, y);
		}
	}
	
	/**
	* Draws a circle.
	* 
	* @param	x		The x position of the center point.
	* @param	y		The y position of the center point.
	* @param	r		The radius from the center to the edge of the circle.
	* 
	* @usage
	* <pre><code>
	* my_mc.beginFill(0x0000FF, 40);
	* my_mc.drawCircle(150, 100, 50);
	* my_mc.endFill();
	* </code></pre>
	*/
	public function drawCircle (x:Number, y:Number, r:Number):Void {
		this.drawOval (x, y, r, r);
	}
	
	/**
	* Draws a curve using the current line style from the current drawing position to (anchorX, anchorY) through the point that ((pointX, pointY) specifies.
	* 
	* @param	pointX		The x position of the curve-through point.
	* @param	pointY		The y position of the curve-through point.
	* @param	anchorX		The x position of the next anchor point.
	* @param	anchorY		The y position of the next anchor point.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(4, 0x0000FF, 60);
	* my_mc.curveThruTo(400,50,100,200);
	* </code></pre>
	*/
	public function curveThruTo (pointX:Number, pointY:Number, anchorX:Number, anchorY:Number):Void {
		var cx:Number = 2*pointX - .5*(this.getXPen() + anchorX);
		var cy:Number = 2*pointY - .5*(this.getYPen() + anchorY);
		this.curveTo (cx, cy, anchorX, anchorY);
	}
	
	/**
	* Draws a curve using the current line style through the three specified points.
	* 
	* @param	p1		The first point of the curve.
	* @param	p2		The middle point of the curve.
	* @param	p3		The third point of the curve.
	* 
	* @usage
	* <pre><code>
	* this.onMouseMove = function() {
	* 	my_mc.clear();
	* 	my_mc.lineStyle(4, 0x0000FF, 60);
	* 	my_mc.drawCurve3Pts({x:100, y:100}, {x:my_mc._xmouse, y:my_mc._ymouse}, {x:400, y:400});
	* }
	* </code></pre>
	*/
	public function drawCurve3Pts (p1:Object, p2:Object, p3:Object):Void {
		this.moveTo (p1.x, p1.y);
		this.curveThruTo(p2.x, p2.y, p3.x, p3.y);
	}
	
	/**
	* Draws a cubic bezier curve using the current line style.
	* 
	* @param	anchorX1	The x position of the first anchor point.
	* @param	anchorY1	The y position of the first anchor point.
	* @param	controlX1	The x position of the first control point.
	* @param	controlY1	The y position of the first control point.
	* @param	controlX2	The x position of the second control point.
	* @param	controlY2	The y position of the second control point.
	* @param	anchorX2	The x position of the second anchor point.
	* @param	anchorY2	The y position of the second anchor point.
	* @param	tolerance	The accuracy of the curve. The smaller the tolerance, the more accurate and complex the curve.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(4, 0xFF0000, 70);
	* my_mc.drawBezier(0, 0, 100, 0, 0, 100, 200, 200, 2);
	* </code></pre>
	*/
	public function drawBezier (anchorX1:Number, anchorY1:Number, controlX1:Number, controlY1:Number, controlX2:Number, controlY2:Number, anchorX2:Number, anchorY2:Number, tolerance:Number):Void {
		if (tolerance == undefined) tolerance = 5;
		this.moveTo (anchorX1, anchorY1);
		this.createBezier (
			{x:anchorX1, y:anchorY1},
			{x:controlX1, y:controlY1},
			{x:controlX2, y:controlY2},
			{x:anchorX2, y:anchorY2},
			tolerance*tolerance );
	}
	
	/**
	* Draws a cubic bezier curve using the current line style.
	* 
	* @param	anchorP1	The first anchor point.
	* @param	controlP1	The first control point.
	* @param	controlP2	The second control point.
	* @param	anchorP2	The second anchor point.
	* @param	tolerance	The accuracy of the curve. The smaller the tolerance, the more accurate and complex the curve.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(4, 0xFF0000, 70);
	* var a:Object = {x:100, y:20};
	* var b:Object = {x:0, y:80};
	* var c:Object = {x:150, y:100};
	* var d:Object = {x:10, y:200};
	* my_mc.drawBezierPts(a, b, c, d, 2);
	* </code></pre>
	*/
	public function drawBezierPts (anchorP1:Object, controlP1:Object, controlP2:Object, anchorP2:Object, tolerance:Number):Void {
		if (tolerance == undefined) tolerance = 5;
		this.moveTo (anchorP1.x, anchorP1.y);
		this.createBezier (anchorP1, controlP1, controlP2, anchorP2, tolerance*tolerance);
	}
	
	/**
	* Draws a cubic bezier curve using the current line style from the current drawing position.
	* 
	* @param	controlX1	The x position of the first control point.
	* @param	controlY1	The y position of the first control point.
	* @param	controlX2	The x position of the second control point.
	* @param	controlY2	The y position of the second control point.
	* @param	anchorX		The x position of the next anchor point.
	* @param	anchorY		The y position of the next anchor point.
	* @param	tolerance	The accuracy of the curve. The smaller the tolerance, the more accurate and complex the curve.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(2, 0x33DD77);
	* my_mc.moveTo(60, 40);
	* my_mc.curveToCubic(100, 0, 0, 100, 250, 30, 2);
	* </code></pre>
	*/
	public function curveToCubic (controlX1:Number, controlY1:Number, controlX2:Number, controlY2:Number, anchorX:Number, anchorY:Number, tolerance:Number):Void {
		if (tolerance == undefined) tolerance = 5;
		this.createBezier (
			{x:this.getXPen(), y:this.getYPen()},
			{x:controlX1, y:controlY1},
			{x:controlX2, y:controlY2},
			{x:anchorX, y:anchorY},
			tolerance*tolerance );
	}
	
	/**
	* Draws a cubic bezier curve using the current line style from the current drawing position.
	* 
	* @param	controlP1	The first control point.
	* @param	controlP2	The second control point.
	* @param	anchorP		The next anchor point.
	* @param	tolerance	The accuracy of the curve. The smaller the tolerance, the more accurate and complex the curve.
	* 
	* @usage
	* <pre><code>
	* my_mc.lineStyle(4, 0x0000FF, 60);
	* my_mc.moveTo(60, 20);
	* var a:Object = {x:20, y:100};
	* var b:Object = {x:90, y:0};
	* var c:Object = {x:200, y:50};
	* my_mc.curveToCubicPts(a, b, c);
	* </code></pre>
	*/
	public function curveToCubicPts (controlP1:Object, controlP2:Object, anchorP:Object, tolerance:Number):Void {
		if (tolerance == undefined) tolerance = 5;
		this.createBezier (
			{x:this.getXPen(), y:this.getYPen()},
			controlP1,
			controlP2,
			anchorP,
			tolerance*tolerance );
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The xpen property of the DrawingClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.xpen);
	* </code></pre>
	*/
	public function get xpen ():Number {
		return this.getXPen();
	}
	public function set xpen (x:Number):Void {
		this.setXPen(x);
	}
	
	/**
	* The ypen property of the DrawingClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.ypen);
	* </code></pre>
	*/
	public function get ypen ():Number {
		return this.getYPen();
	}
	public function set ypen (y:Number):Void {
		this.setYPen(y);
	}
	
	/**
	* The xpenStart property of the DrawingClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.xpenStart);
	* </code></pre>
	*/
	public function get xpenStart ():Number {
		return this.getXPenStart();
	}
	
	/**
	* The ypenStart property of the DrawingClip.
	* 
	* @usage
	* <pre><code>
	* trace (my_mc.ypenStart);
	* </code></pre>
	*/
	public function get ypenStart ():Number {
		return this.getYPenStart();
	}
	
}